@@ -29,7 +29,9 @@ setMethod("show", "dictionary",
 #' 
 #' Create a quanteda dictionary, either from a list or by importing from a 
 #' foreign format.  Currently supported input file formats are the Wordstat and 
-#' LIWC formats.
+#' LIWC formats.  The import using the LIWC format works with all currently
+#' available dictionary files supplied as part of the LIWC 2001, 2007, and 2015
+#' software (see References).
 #' @param x a list of character vector dictionary entries, including regular 
 #'   expressions (see examples)
 #' @param file file identifier for a foreign dictionary
@@ -38,10 +40,10 @@ setMethod("show", "dictionary",
 #'   Provalis Research's Wordstat software} \item{\code{"LIWC"}}{format used by 
 #'   the Linguistic Inquiry and Word Count software} }
 #' @param concatenator the character in between multi-word dictionary values. 
-#'   This defaults to \code{"_"} except LIWC-formatted files, which defaults to
+#'   This defaults to \code{"_"} except LIWC-formatted files, which defaults to 
 #'   a single space \code{" "}.
-#' @param encoding additional optional encoding value for reading in imported
-#'   dictionaries. This uses the \link{iconv} labels for encoding.  See the
+#' @param encoding additional optional encoding value for reading in imported 
+#'   dictionaries. This uses the \link{iconv} labels for encoding.  See the 
 #'   "Encoding" section of the help for \link{file}.
 #' @param toLower if \code{TRUE}, convert all dictionary values to lowercase
 #' @return A dictionary class object, essentially a specially classed named list
@@ -226,10 +228,21 @@ readLIWCdict <- function(path, toLower = TRUE, encoding = getOption("encoding"))
         catlist <- gsub("\\(.+\\)", "", catlist)
     }
         
+    ## clean up irregular dictionary files
+    # remove any repeated \t
+    catlist <- gsub("\t\t+", "\t", catlist)
+    # remove any spaced before a \t
+    catlist <- gsub(" +\t", "\t", catlist)
+    # replace any blanks that should be \t with \t (e.g. in Moral Foundations dictionary)
+    catlist <- gsub("(\\d+) +(\\d+)", "\\1\t\\2", catlist)
+    # remove any \t only lines or empty lines
+    if (length(blanklines <- grep("^\\s*$", catlist))) 
+        catlist <- catlist[-blanklines]
+
     catlist <- strsplit(catlist, "\t")
     # catlist <- tokenize(catlist, what = "fasterword", removeNumbers = FALSE)
     catlist <- as.data.frame(do.call(rbind, lapply(catlist, '[', 1:max(sapply(catlist, length)))), stringsAsFactors = FALSE)
-    catlist[, 2:ncol(catlist)] <- lapply(catlist[2:ncol(catlist)], as.integer)
+    catlist[, 2:ncol(catlist)] <- suppressWarnings(apply(catlist[, 2:ncol(catlist)], 2, as.integer))
     names(catlist)[1] <- "category"
     if (toLower) catlist$category <- toLower(catlist$category)
     # remove any blank rows
@@ -253,7 +266,7 @@ readLIWCdict <- function(path, toLower = TRUE, encoding = getOption("encoding"))
         terms[[i]] <- as.numeric(catlist[i, !is.na(catlist[i,])])
     }
     
-    for(ind in 1:length(terms)){
+    for (ind in 1:length(terms)) {
         for(num in as.numeric(terms[[ind]])){
             thisCat <- guide$catName[which(guide$catNum==num)]
             thisTerm <- names(terms[ind])
