@@ -180,23 +180,91 @@ selectFeatures.dfm <- function(x, features, selection = c("keep", "remove"),
         return(x[, -featIndex])
 }
 
-
 #' @rdname selectFeatures
-#' @examples
-#' toks <- tokenize(c("This is some example text from me.", "More of the example text."), 
-#'                  removePunct = TRUE)
-#' selectFeatures(toks, stopwords("english"), "remove")
-#' selectFeatures(toks, "ex", "keep", valuetype = "regex")
+#' @param padding (only for \code{tokenizedTexts} objects) if \code{TRUE}, leave
+#'   an empty string where the removed tokens previously existed.  This is
+#'   useful if a positional match is needed between the pre- and post-selected
+#'   features, for instance if a window of adjacency needs to be computed.
+#' @param indexing use dfm-based index to efficiently process large tokenizedTexts object
 #' @export
+#' @examples 
+#' \dontrun{## performance comparisons
+#' data(SOTUCorpus, package = "quantedaData")
+#' toks <- tokenize(SOTUCorpus, removePunct = TRUE)
+#' # toks <- tokenize(tokenize(SOTUCorpus, what='sentence', simplify = TRUE), removePunct = TRUE)
+#' # head to head, old v. new
+#' system.time(selectFeaturesOLD(toks, stopwords("english"), "remove", verbose = FALSE))
+#' system.time(selectFeatures(toks, stopwords("english"), "remove", verbose = FALSE))
+#' system.time(selectFeaturesOLD(toks, c("and", "of"), "remove", verbose = FALSE, valuetype = "regex"))
+#' system.time(selectFeatures(toks, c("and", "of"), "remove", verbose = FALSE, valuetype = "regex"))
+#' microbenchmark::microbenchmark(
+#'     old = selectFeaturesOLD(toks, stopwords("english"), "remove", verbose = FALSE),
+#'     new = selectFeatures(toks, stopwords("english"), "remove", verbose = FALSE),
+#'     times = 5, unit = "relative")
+#' microbenchmark::microbenchmark(
+#'     new = selectFeaturesOLD(toks, c("and", "of"), "remove", verbose = FALSE, valuetype = "regex"),
+#'     old = selectFeatures(toks, c("and", "of"), "remove", verbose = FALSE, valuetype = "regex"),
+#'     times = 2, unit = "relative")
+#'     
+#' types <- unique(unlist(toks))
+#' numbers <- types[stringi::stri_detect_regex(types, '[0-9]')]
+#' microbenchmark::microbenchmark(
+#'     new = selectFeaturesOLD(toks, numbers, "remove", verbose = FALSE, valuetype = "fixed"),
+#'     old = selectFeatures(toks, numbers, "remove", verbose = FALSE, valuetype = "fixed"),
+#'     times = 2, unit = "relative")  
+#'     
+#' # removing tokens before dfm, versus after
+#' microbenchmark::microbenchmark(
+#'     pre = dfm(selectFeaturesOLD(toks, stopwords("english"), "remove"), verbose = FALSE),
+#'     post = dfm(toks, ignoredFeatures = stopwords("english"), verbose = FALSE),
+#'     times = 5, unit = "relative")
+#' }
+#' 
+#' ## with simple examples
+#' toks <- tokenize(c("This is a sentence.", "This is a second sentence."), 
+#'                  removePunct = TRUE)
+#' selectFeatures(toks, c("is", "a", "this"), selection = "remove", 
+#'                 valuetype = "fixed", padding = TRUE, case_insensitive = TRUE)
+#' 
+#' # how case_insensitive works
+#' selectFeatures(toks, c("is", "a", "this"), selection = "remove", 
+#'                valuetype = "fixed", padding = TRUE, case_insensitive = FALSE)
+#' selectFeatures(toks, c("is", "a", "this"), selection = "remove", 
+#'                valuetype = "fixed", padding = TRUE, case_insensitive = TRUE)
+#' selectFeatures(toks, c("is", "a", "this"), selection = "remove", 
+#'                valuetype = "glob", padding = TRUE, case_insensitive = TRUE)
+#' selectFeatures(toks, c("is", "a", "this"), selection = "remove", 
+#'                valuetype = "glob", padding = TRUE, case_insensitive = FALSE)
+#' 
+#' # with longer texts
+#' txts <- c(exampleString, inaugTexts[2])
+#' toks <- tokenize(txts)
+#' selectFeatures(toks, stopwords("english"), "remove")
+#' selectFeatures(toks, stopwords("english"), "keep")
+#' selectFeatures(toks, stopwords("english"), "remove", padding = TRUE)
+#' selectFeatures(toks, stopwords("english"), "keep", padding = TRUE)
+#' selectFeatures(tokenize(encodedTexts[1]), stopwords("english"), "remove", padding = TRUE)
 selectFeatures.tokenizedTexts <- function(x, features, selection = c("keep", "remove"), 
-                                          valuetype = c("glob", "regex", "fixed"),
-                                          case_insensitive = TRUE,
-                                          verbose = TRUE, ...) {
+                                           valuetype = c("glob", "regex", "fixed"),
+                                           case_insensitive = TRUE, padding = FALSE, indexing = FALSE,
+                                           verbose = FALSE, ...) {
     selection <- match.arg(selection)
     valuetype <- match.arg(valuetype)
-    features <- unique(unlist(features))  # to convert any dictionaries
-    
     originalvaluetype <- valuetype
+    features <- unique(unlist(features, use.names=FALSE))  # to convert any dictionaries
+    y <- deepcopy(x) # copy x to y to prevent changes in x
+    n <- length(y)
+    
+    if(indexing){
+        if(verbose) cat("Indexing tokens...\n")
+        index <- dfm(y, verbose = FALSE)
+        index_binary <- as(index, 'nMatrix')
+        types <- colnames(index_binary)
+    }else{
+        types <- unique(unlist(y, use.names=FALSE))
+        flag <- rep(TRUE, n)
+    }
+    
     # convert glob to fixed if no actual glob characters (since fixed is much faster)
     if (valuetype == "glob") {
         # treat as fixed if no glob characters detected
@@ -209,45 +277,35 @@ selectFeatures.tokenizedTexts <- function(x, features, selection = c("keep", "re
     }
     
     if (valuetype == "fixed") {
+        
         if (case_insensitive) {
-            features <- toLower(features)
-            x <- toLower(x)
-        }
-        if (selection == "remove") 
-            result <- lapply(x, function(x) x[which(!(x %in% features))])
-        else 
-            result <- lapply(x, function(x) x[which((x %in% features))])
-    } else if (valuetype == "regex") {
-        if (selection == "remove") {
-            result <- lapply(x, function(y) {
-                removeIndex <- which(stringi::stri_detect_regex(y, rep(paste0(features, collapse = "|"), length(y)), 
-                                                                case_insensitive = case_insensitive, ...))
-                if (length(removeIndex)) y[-removeIndex] else y
-            })
+            #types <- unique(unlist(y, use.names=FALSE))
+            types_match <- types[toLower(types) %in% toLower(features)]
         } else {
-            result <- lapply(x, function(y) y[which(stringi::stri_detect_regex(y, rep(paste0(features, collapse = "|"), length(y)), 
-                                                                               case_insensitive = case_insensitive, ...))])
+            types_match <- features
+        }
+        if (indexing) flag <- Matrix::rowSums(index_binary[,types_match]) > 0 # identify texts where types match appear
+        if (verbose) cat(sprintf("Scanning %.2f%% of texts...\n", 100 * sum(flag) / n))
+        if(selection == "remove"){
+            select_tokens_cppl(y, flag, types_match, TRUE, padding)
+        }else{ 
+            select_tokens_cppl(y, flag, types_match, FALSE, padding)
+        }
+    } else if (valuetype == "regex") {
+        if (verbose) cat("Converting regex to fixed...\n")
+        types_match <- regex2fixed(features, types, case_insensitive) # get all the unique types that match regex
+        if(indexing) flag <- Matrix::rowSums(index_binary[,types_match]) > 0 # identify texts where types match appear
+        if(verbose) cat(sprintf("Scanning %.2f%% of texts...\n", 100 * sum(flag) / n))
+        if (selection == "remove") {
+            select_tokens_cppl(y, flag, types_match, TRUE, padding)  # search as fixed
+        } else {
+            select_tokens_cppl(y, flag, types_match, FALSE, padding) # search as fixed
         }
     }
     
-    class(result) <- c("tokenizedTexts", class(result))
-    attributes(result) <- attributes(x)
-    result
-    #     if (verbose & !features_from_dfm) 
-    #         cat(ifelse(selection=="keep", "kept", "removed"), " ", 
-    #             format(length(featIndex), big.mark=","),
-    #             " feature", ifelse(length(featIndex) > 1 | length(featIndex)==0, "s", ""), 
-    #             ", from ", length(features), " supplied (", originalvaluetype, ") feature type",
-    #             ifelse(length(features) > 0 | length(featIndex)==0, "s", ""),
-    #             "\n", sep = "")
-    #     if (verbose & features_from_dfm)
-    #         cat(ifelse(selection=="keep", "found", "zeroed"), " ", 
-    #             format(length(featIndex), big.mark=","),
-    #             " feature", ifelse(length(featIndex) > 1 | length(featIndex)==0, "s", ""), 
-    #             " from ", length(features), " supplied type",
-    #             ifelse(length(features) > 0 | length(featIndex)==0, "s", ""),
-    #             " in a dfm,", sep = "")
-    
+    class(y) <- c("tokenizedTexts", class(x))
+    attributes(y) <- attributes(x)
+    return(y)
 }
 
 #' @rdname selectFeatures
@@ -458,4 +516,16 @@ selectFeatures_collocations <- function(x, features, selection = c("keep", "remo
 #                          unit = "relative", times = 30
 #     )
 
+# Convert regex to fixed
+# @param regex regular expression
+# @param types unique types of tokens
+# @param case_insensitive case sensitivity
+# @param ... additional arguments passed to \code{\link[stringi]{stri_detect_regex}}
+# @export
+regex2fixed <- function(regex, types, case_insensitive = TRUE, ...){
+    regex_joined <- paste0(regex, collapse = "|")
+    types_match <- types[stringi::stri_detect_regex(types, regex_joined, case_insensitive = case_insensitive, ...)]  
+    return(types_match)
+}
+
 
